@@ -21,16 +21,19 @@
 */
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
 #include <string.h>
+#include <libgnome/gnome-i18n.h>
+#include <glade/glade.h>
+
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
-#include "htmltext.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-fontstyle.h"
 #include "htmlengine-edit-cut-and-paste.h"
+#include "htmlimage.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
+#include "htmltext.h"
 
 #include "properties.h"
 #include "link.h"
@@ -38,21 +41,67 @@
 
 struct _GtkHTMLEditLinkProperties {
 	GtkHTMLControlData *cd;
-	GtkWidget *entry_text;
+	GtkWidget *entry_description;
+	GtkWidget *label_description;
 	GtkWidget *entry_url;
 
-	HTMLText *text;
+	HTMLObject *object;
 	gint offset;
+	gboolean selection;
+	gboolean description_empty;
+	gboolean insert;
+	int insert_start_offset;
+	int insert_end_offset;
+	HTMLObject *insert_object;
 
-	gboolean url_changed;
+	gboolean disable_change;
 };
 typedef struct _GtkHTMLEditLinkProperties GtkHTMLEditLinkProperties;
 
 static void
-changed (GtkWidget *w, GtkHTMLEditLinkProperties *data)
+url_changed (GtkWidget *w, GtkHTMLEditLinkProperties *d)
 {
-	data->url_changed = TRUE;
-	gtk_html_edit_properties_dialog_change (data->cd->properties_dialog);
+	const char *url, *desc;
+
+	if (d->disable_change)
+		return;
+
+	url = gtk_entry_get_text (GTK_ENTRY (d->entry_url));
+	desc = gtk_entry_get_text (GTK_ENTRY (d->entry_description));
+	if (d->selection)
+		html_engine_set_link (d->cd->html->engine, url);
+	else {
+		if (!desc || !*desc || d->description_empty) {
+			gtk_entry_set_text (GTK_ENTRY (d->entry_description), url);
+			d->description_empty = TRUE;
+		}
+	}
+}
+
+static void
+description_changed (GtkWidget *w, GtkHTMLEditLinkProperties *d)
+{
+	HTMLEngine *e = d->cd->html->engine;
+	const char *text;
+	int len;
+
+	d->description_empty = FALSE;
+
+	if (d->disable_change || !editor_has_html_object (d->cd, d->insert_object))
+		return;
+
+	html_cursor_jump_to (e->cursor, e, d->insert_object, d->insert_start_offset);
+	html_engine_set_mark (e);
+	html_cursor_jump_to (e->cursor, e, d->insert_object, d->insert_end_offset);
+	html_engine_delete (e);
+	text = gtk_entry_get_text (GTK_ENTRY (w));
+	if (text && *text) {
+		len = g_utf8_strlen (text, -1);
+		html_engine_paste_link (e, text, len, gtk_entry_get_text (GTK_ENTRY (d->entry_url)));
+		d->insert_object = e->cursor->object;
+	} else
+		len = 0;
+	d->insert_end_offset = d->insert_start_offset + len;
 }
 
 static void
@@ -65,66 +114,108 @@ test_clicked (GtkWidget *w, GtkHTMLEditLinkProperties *data)
 }
 
 static void
-set_ui (GtkHTMLEditLinkProperties *data)
+link_set_ui (GtkHTMLEditLinkProperties *d)
 {
+	HTMLEngine *e = d->cd->html->engine;
 	gchar *url, *link_text;
 
-	link_text = html_text_get_link_text (data->text, data->offset);
+	d->disable_change = TRUE;
+
+	if (html_engine_is_selection_active (e)) {
+		d->selection = TRUE;
+
+		gtk_widget_hide (d->label_description);
+		gtk_widget_hide (d->entry_description);
+	} else {
+		char *url = NULL;
+
+		if (HTML_IS_TEXT (e->cursor->object))
+			url = html_object_get_complete_url (e->cursor->object, e->cursor->offset);
+
+		d->selection = FALSE;
+		d->insert = TRUE;
+		d->insert_object = e->cursor->object;
+
+		if (url) {
+			gtk_entry_set_text (GTK_ENTRY (d->entry_url), url);
+			gtk_widget_hide (d->label_description);
+			gtk_widget_hide (d->entry_description);
+		
+			if (HTML_IS_IMAGE (d->insert_object)) {
+				d->insert_start_offset = 0;
+				d->insert_end_offset = 1;
+			} else {
+				Link *link;
+				link = html_text_get_link_at_offset (HTML_TEXT (d->insert_object), e->cursor->offset);
+
+				if (link) {
+					d->insert_start_offset = link->start_offset;
+					d->insert_end_offset = link->end_offset;
+				}
+			}
+		} else {
+			gtk_entry_set_text (GTK_ENTRY (d->entry_url), "http://");
+
+			d->insert_start_offset = d->insert_end_offset = e->cursor->offset;
+		}
+	}
+
+	d->disable_change = FALSE;
+
+	/* link_text = html_text_get_link_text (data->text, data->offset);
 	gtk_entry_set_text (GTK_ENTRY (data->entry_text), link_text);
 	g_free (link_text);
 
 	url = html_object_get_complete_url (HTML_OBJECT (data->text), data->offset);
 	gtk_entry_set_text (GTK_ENTRY (data->entry_url), url ? url : "");
-	g_free (url);
+	g_free (url); */
 }
 
-static gboolean stock_test_url_added = FALSE;
-#define GTKHTML_STOCK_TEST_URL "gtkhtml-stock-test-url"
-static GtkStockItem test_url_items [] =
+static void
+test_url_clicked (GtkWidget *w, GtkHTMLEditLinkProperties *d)
 {
-	{ GTKHTML_STOCK_TEST_URL, N_("Test URL..."), 0, 0, NULL }
-};
+	const char *url = gtk_entry_get_text (GTK_ENTRY (d->entry_url));
+
+	if (url)
+		gnome_url_show (url, NULL);
+}
 
 static GtkWidget *
-link_widget (GtkHTMLEditLinkProperties *data, gboolean insert)
+link_widget (GtkHTMLEditLinkProperties *d, gboolean insert)
 {
-	GtkHTMLControlData *cd = data->cd;
-	GtkWidget *vbox, *hbox, *button, *frame, *f1;
+	GtkHTMLControlData *cd = d->cd;
+	GtkWidget *link_page, *button;
+	GladeXML *xml;
 
-	if (!stock_test_url_added) {
-		GdkPixbuf *pixbuf;
-		GError *error = NULL;
+	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "link_page", NULL);
+	if (!xml)
+		g_error (_("Could not load glade file."));
 
-		pixbuf = gdk_pixbuf_new_from_file (ICONDIR "/insert-link-16.png", &error);
-		if (!pixbuf) {
-			g_error_free (error);
-		} else {
-			GtkIconSet *test_url_iconset = gtk_icon_set_new_from_pixbuf (pixbuf);
+	link_page = glade_xml_get_widget (xml, "link_page");
 
-			if (test_url_iconset) {
-				GtkIconFactory *factory = gtk_icon_factory_new ();
+	editor_check_stock ();
+	button = gtk_button_new_from_stock (GTKHTML_STOCK_TEST_URL);
+	g_signal_connect (button, "clicked", G_CALLBACK (test_url_clicked), d);
+	gtk_widget_show (button);
+	gtk_table_attach (GTK_TABLE (glade_xml_get_widget (xml, "table_link")), button, 2, 3, 0, 1, 0, 0, 0, 0);
 
-				gtk_icon_factory_add (factory, GTKHTML_STOCK_TEST_URL, test_url_iconset);
-				gtk_icon_factory_add_default (factory);
-			}
-			gtk_stock_add_static (test_url_items, G_N_ELEMENTS (test_url_items));
-		}
-		stock_test_url_added = TRUE;
-	}
+	d->entry_url = glade_xml_get_widget (xml, "entry_url");
+	g_signal_connect (d->entry_url, "changed", G_CALLBACK (url_changed), d);
 
-	vbox = gtk_vbox_new (FALSE, 6);
+	d->entry_description = glade_xml_get_widget (xml, "entry_description");
+	g_signal_connect (d->entry_description, "changed", G_CALLBACK (description_changed), d);
+
+	d->label_description = glade_xml_get_widget (xml, "label_description");
+
+	/*GtkWidget *vbox, *hbox, *button, *frame, *f1;
+
+	vbox = gtk_vbox_new (FALSE, 18);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 
 	data->entry_text = gtk_entry_new ();
 	data->entry_url  = gtk_entry_new ();
 
-	frame = gtk_frame_new (_("Link text"));
-	f1    = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (f1), GTK_SHADOW_NONE);
-	gtk_container_set_border_width (GTK_CONTAINER (f1), 6);
-	gtk_container_add (GTK_CONTAINER (f1), data->entry_text);
-	gtk_container_add (GTK_CONTAINER (frame), f1);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor_hig_vbox (_("Link Text"), data->entry_text), FALSE, FALSE, 0);
 
 	if (html_engine_is_selection_active (cd->html->engine)) {
 		gchar *str;
@@ -134,17 +225,12 @@ link_widget (GtkHTMLEditLinkProperties *data, gboolean insert)
 		g_free (str);
 	}
 
-	frame = gtk_frame_new (_("Click will follow this URL"));
-	f1    = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (f1), GTK_SHADOW_NONE);
-	gtk_container_set_border_width (GTK_CONTAINER (f1), 6);
-	hbox = gtk_hbox_new (FALSE, 12);
+	hbox = gtk_hbox_new (FALSE, 5);
+	editor_check_stock ();
 	button = gtk_button_new_from_stock (GTKHTML_STOCK_TEST_URL);
 	gtk_box_pack_start (GTK_BOX (hbox), data->entry_url, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (f1), hbox);
-	gtk_container_add (GTK_CONTAINER (frame), f1);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor_hig_vbox (_("Click Will Follow This URL"), hbox), FALSE, FALSE, 0);
 
 	if (!insert) {
 		gtk_widget_set_sensitive (data->entry_text, FALSE);
@@ -153,17 +239,18 @@ link_widget (GtkHTMLEditLinkProperties *data, gboolean insert)
 
 	g_signal_connect (data->entry_text, "changed", G_CALLBACK (changed), data);
 	g_signal_connect (data->entry_url, "changed", G_CALLBACK (changed), data);
-	g_signal_connect (button, "clicked", G_CALLBACK (test_clicked), data);
+	g_signal_connect (button, "clicked", G_CALLBACK (test_clicked), data);*/
 
-	gtk_widget_show_all (vbox);
+	gtk_widget_show_all (link_page);
+	link_set_ui (d);
 
-	return vbox;
+	return link_page;
 }
 
 GtkWidget *
 link_insert (GtkHTMLControlData *cd, gpointer *set_data)
 {
-	GtkHTMLEditLinkProperties *data = g_new (GtkHTMLEditLinkProperties, 1);
+	GtkHTMLEditLinkProperties *data = g_new0 (GtkHTMLEditLinkProperties, 1);
 
 	*set_data = data;
 	data->cd = cd;
@@ -174,15 +261,15 @@ link_insert (GtkHTMLControlData *cd, gpointer *set_data)
 GtkWidget *
 link_properties (GtkHTMLControlData *cd, gpointer *set_data)
 {
-	GtkHTMLEditLinkProperties *data = g_new (GtkHTMLEditLinkProperties, 1);
+	GtkHTMLEditLinkProperties *data = g_new0 (GtkHTMLEditLinkProperties, 1);
 
 	g_return_val_if_fail (cd->html->engine->cursor->object, NULL);
 	g_return_val_if_fail (HTML_IS_LINK_TEXT (cd->html->engine->cursor->object), NULL);
 
 	*set_data = data;
 	data->cd = cd;
-	data->text = HTML_TEXT (cd->html->engine->cursor->object);
-	data->offset = cd->html->engine->cursor->offset;
+	//data->text = HTML_TEXT (cd->html->engine->cursor->object);
+	//data->offset = cd->html->engine->cursor->offset;
 
 	return link_widget (data, FALSE);
 }
@@ -196,7 +283,7 @@ link_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 	const gchar *url;
 	gchar *target;
 
-	if (data->url_changed) {
+	/* if (data->url_changed) {
 		gchar *url_copy;
 		gint position;
 
@@ -224,7 +311,7 @@ link_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 		html_engine_update_insertion_url_and_target (e);
 		g_free (url_copy);
 		html_cursor_jump_to_position (e->cursor, e, position);
-	}
+		} */
 
 	return TRUE;
 }
@@ -238,7 +325,7 @@ link_insert_cb (GtkHTMLControlData *cd, gpointer get_data)
 	gchar *target;
 	const gchar *text;
 
-	url  = gtk_entry_get_text (GTK_ENTRY (data->entry_url));
+	/* url  = gtk_entry_get_text (GTK_ENTRY (data->entry_url));
 	text = gtk_entry_get_text (GTK_ENTRY (data->entry_text));
 	if (url && text && *url && *text) {
 		HTMLObject *new_text;
@@ -251,12 +338,29 @@ link_insert_cb (GtkHTMLControlData *cd, gpointer get_data)
 					  html_colorset_get_color (e->settings->color_set, HTMLLinkColor));
 		html_text_add_link (HTML_TEXT (new_text), e, url_copy, target, 0, HTML_TEXT (new_text)->text_len);
 
-		html_engine_paste_object (e, new_text, g_utf8_strlen (text, -1));
+		html_engine_paste_object (e, new_text, HTML_TEXT (new_text)->text_len);
 
 		g_free (url_copy);
-	}
 
-	return TRUE;
+		return TRUE;
+	} else {
+		GtkWidget *toplevel;
+		GtkWidget *dialog;
+
+		toplevel = gtk_widget_get_toplevel (cd->properties_dialog->dialog);
+		if (!GTK_WIDGET_TOPLEVEL (toplevel))
+			toplevel = NULL;
+
+		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+						 _("Please fill 'Link text' and 'URL' entries"));
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		} */
+
+	return FALSE;
 }
 
 void
