@@ -100,12 +100,11 @@ insert_link (GtkWidget *mi, GtkHTMLControlData *cd)
 	if (cd->properties_dialog)
 		gtk_html_edit_properties_dialog_close (cd->properties_dialog);
 
-	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, TRUE, _("Insert"), ICONDIR "/insert-link-24.png");
+	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, _("Insert"), ICONDIR "/insert-link-24.png");
 
 	gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 						   GTK_HTML_EDIT_PROPERTY_LINK, _("Link"),
 						   link_insert,
-						   link_insert_cb,
 						   link_close_cb);
 
 	gtk_html_edit_properties_dialog_show (cd->properties_dialog);
@@ -184,7 +183,8 @@ show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 
 	if (cd->properties_dialog)
 		gtk_html_edit_properties_dialog_close (cd->properties_dialog);
-	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"), ICONDIR "/properties-16.png");
+	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, _("Properties"),
+								     gnome_icon_theme_lookup_icon (cd->icon_theme, "stock_properties", 16, NULL, NULL));
 
 	cur = cd->properties_types;
 	while (cur) {
@@ -194,56 +194,48 @@ show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Text"),
 								   text_properties,
-								   text_apply_cb,
 								   text_close_cb);
 			break;
 		case GTK_HTML_EDIT_PROPERTY_LINK:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Link"),
 								   link_properties,
-								   link_apply_cb,
 								   link_close_cb);
 			break;
 		case GTK_HTML_EDIT_PROPERTY_IMAGE:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Image"),
 								   image_properties,
-								   image_apply_cb,
 								   image_close_cb);
 								   break;
 		case GTK_HTML_EDIT_PROPERTY_PARAGRAPH:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Paragraph"),
 								   paragraph_properties,
-								   paragraph_apply_cb,
 								   paragraph_close_cb);
 			break;
 		case GTK_HTML_EDIT_PROPERTY_BODY:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Page"),
 								   body_properties,
-								   body_apply_cb,
 								   body_close_cb);
 			break;
 		case GTK_HTML_EDIT_PROPERTY_RULE:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Rule"),
 								   rule_properties,
-								   rule_apply_cb,
 								   rule_close_cb);
 			break;
 		case GTK_HTML_EDIT_PROPERTY_TABLE:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Table"),
 								   table_properties,
-								   table_apply_cb,
 								   table_close_cb);
 			break;
 		case GTK_HTML_EDIT_PROPERTY_CELL:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Cell"),
 								   cell_properties,
-								   cell_apply_cb,
 								   cell_close_cb);
 			break;
 		default:
@@ -269,12 +261,11 @@ link_prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
 	if (cd->properties_dialog)
 		gtk_html_edit_properties_dialog_close (cd->properties_dialog);
 
-	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"), ICONDIR "/insert-link-24.png");
+	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, _("Properties"), ICONDIR "/insert-link-24.png");
 
 	gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 						   GTK_HTML_EDIT_PROPERTY_LINK, _("Link"),
 						   link_properties,
-						   link_apply_cb,
 						   link_close_cb);
 
 	gtk_html_edit_properties_dialog_show (cd->properties_dialog);
