@@ -44,6 +44,7 @@
 #include "htmltext.h"
 #include "htmlrule.h"
 #include "htmltype.h"
+#include "htmlsettings.h"
 
 #include "gtkhtmldebug.h"
 
@@ -313,10 +314,13 @@ static GdkColor *
 get_bg_color (HTMLObject *o,
 	      HTMLPainter *p)
 {
-	
-	return o->parent
-		? html_object_get_bg_color (o->parent, p)
-		: &((html_colorset_get_color (p->color_set, HTMLBgColor))->color);
+	if (o->parent)
+		return html_object_get_bg_color (o->parent, p);	
+
+	if (p->widget && GTK_IS_HTML (p->widget))
+		return &((html_colorset_get_color (GTK_HTML (p->widget)->engine->settings->color_set,
+						   HTMLBgColor))->color);
+	return NULL;
 }
 
 static HTMLObject*
@@ -541,8 +545,7 @@ save_plain (HTMLObject *self,
 }
 
 static gint
-check_page_split (HTMLObject *self,
-		  gint y)
+check_page_split (HTMLObject *self, HTMLPainter *p, gint y)
 {
 	return 0;
 }
@@ -1218,12 +1221,11 @@ html_object_save_plain (HTMLObject *self,
 }
 
 gint
-html_object_check_page_split  (HTMLObject *self,
-			       gint y)
+html_object_check_page_split  (HTMLObject *self, HTMLPainter *p, gint y)
 {
 	g_return_val_if_fail (self != NULL, 0);
 
-	return (* HO_CLASS (self)->check_page_split) (self, y);
+	return (* HO_CLASS (self)->check_page_split) (self, p, y);
 }
 
 void
