@@ -82,6 +82,7 @@ html_text_pango_info_new (gint n)
 	pi = g_new (HTMLTextPangoInfo, 1);
 	pi->n = n;
 	pi->entries = g_new0 (HTMLTextPangoInfoEntry, n);
+	pi->attrs = NULL;
 
 	return pi;
 }
@@ -93,9 +94,9 @@ html_text_pango_info_destroy (HTMLTextPangoInfo *pi)
 
 	for (i = 0; i < pi->n; i ++) {
 		pango_item_free (pi->entries [i].item);
-		g_free (pi->entries [i].attrs);
 		g_free (pi->entries [i].widths);
 	}
+	g_free (pi->attrs);
 	g_free (pi);
 }
 
@@ -340,6 +341,9 @@ html_text_op_copy_helper (HTMLText *text, GList *from, GList *to, guint *len)
 	g_free (rvt->text);
 	rvt->text = nt;
 
+	rvt->spell_errors = remove_spell_errors (rvt->spell_errors, 0, begin);
+	rvt->spell_errors = remove_spell_errors (rvt->spell_errors, end, text->text_len - end);
+
 	if (end_index < text->text_bytes)
 		cut_attr_list (rvt, end_index, text->text_bytes);
 	if (begin_index > 0)
@@ -904,16 +908,18 @@ html_text_get_attr_list (HTMLText *text, gint start_index, gint end_index)
 
 void
 html_text_calc_text_size (HTMLText *t, HTMLPainter *painter,
-			  const gchar *text,
-			  guint len, HTMLTextPangoInfo *pi, GList *glyphs, gint start_byte_offset, gint *line_offset,
+			  gint start_byte_offset,
+			  guint len, HTMLTextPangoInfo *pi, GList *glyphs, gint *line_offset,
 			  GtkHTMLFontStyle font_style,
 			  HTMLFontFace *face,
 			  gint *width, gint *asc, gint *dsc)
 {
 		PangoAttrList *attrs = NULL;
+		char *text = t->text + start_byte_offset;
 
-		if (HTML_IS_PRINTER (painter))
+		if (HTML_IS_PRINTER (painter)) {
 			attrs = html_text_get_attr_list (t, start_byte_offset, start_byte_offset + (g_utf8_offset_to_pointer (text, len) - text));
+		}
 		
 		html_painter_calc_text_size (painter, text, len, pi, attrs, glyphs,
 					     start_byte_offset, line_offset, font_style, face, width, asc, dsc);
@@ -979,7 +985,7 @@ html_text_calc_part_width (HTMLText *text, HTMLPainter *painter, char *start, gi
 		}
 		width = PANGO_PIXELS (width);
 	} else {
-		html_text_calc_text_size (text, painter, start, len, NULL, NULL, 0, &line_offset,
+		html_text_calc_text_size (text, painter, start - text->text, len, NULL, NULL, &line_offset,
 					  html_text_get_font_style (text), text->face, &width, asc, dsc);
 	}
 
@@ -996,6 +1002,7 @@ calc_preferred_width (HTMLObject *self,
 	text = HTML_TEXT (self);
 
 	width = html_text_calc_part_width (text, painter, text->text, 0, text->text_len, &self->ascent, &self->descent);
+	self->y = self->ascent;
 	if (html_clueflow_tabs (HTML_CLUEFLOW (self->parent), painter)) {
 		gint line_offset;
 		gint tabs;
@@ -1133,6 +1140,24 @@ html_text_add_cite_color (PangoAttrList *attrs, HTMLText *text, HTMLClueFlow *fl
 	}
 }
 
+void
+html_text_remove_link_line_breaks (HTMLText *text)
+{
+	GSList *cur;
+
+	if (!text->pi || !text->pi->attrs)
+		return;
+
+	for (cur = text->links; cur; cur = cur->next) {
+		Link *link = (Link *) cur->data;
+		int offset;
+
+		for (offset = link->start_offset; offset < link->end_offset; offset ++)
+			if (offset > 0 && !text->pi->attrs [offset - 1].is_white)
+				text->pi->attrs [offset].is_line_break = 0;
+	}
+}
+
 HTMLTextPangoInfo *
 html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 {
@@ -1150,7 +1175,7 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 		HTMLClueFlow *flow = NULL;
 		HTMLEngine *e = NULL;
 		gchar *translated, *heap = NULL;
-		gint i;
+		int i, offset;
 
 		if (text->text_bytes > HTML_ALLOCA_MAX)
 			heap = translated = g_malloc (text->text_bytes);
@@ -1211,13 +1236,15 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 			}
 		}
 
-		if (text->select_length) {
+		if (e && text->select_length) {
 			gchar *end;
 			gchar *start;
 			GdkColor fg = html_colorset_get_color_allocated
-				(painter, painter->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color;
+				(e->settings->color_set, painter,
+				 painter->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color;
 			GdkColor bg = html_colorset_get_color_allocated
-				(painter, painter->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color;
+				(e->settings->color_set, painter,
+				 painter->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color;
 			
 			start = html_text_get_text (text,  text->select_start);
 			end = g_utf8_offset_to_pointer (start, text->select_length);
@@ -1238,16 +1265,43 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 
 		text->pi = html_text_pango_info_new (g_list_length (items));
 
-		for (i = 0, cur = items; i < text->pi->n; i ++, cur = cur->next) {
+		for (i = 0, cur = items; i < text->pi->n; i ++, cur = cur->next)
+			text->pi->entries [i].item = (PangoItem *) cur->data;
+
+		offset = 0;
+		text->pi->attrs = g_new (PangoLogAttr, text->text_len + 1);
+		for (i = 0; i < text->pi->n; i ++) {
+			PangoItem tmp_item;
+			PangoLogAttr *attrs;
+			int start_i, start_offset;
+
+			start_i = i;
+			start_offset = offset;
+			offset += text->pi->entries [i].item->num_chars;
+			tmp_item = *text->pi->entries [i].item;
+			while (i < text->pi->n - 1) {
+				if (tmp_item.analysis.lang_engine == text->pi->entries [i + 1].item->analysis.lang_engine) {
+					tmp_item.length += text->pi->entries [i + 1].item->length;
+					tmp_item.num_chars += text->pi->entries [i + 1].item->num_chars;
+					offset += text->pi->entries [i + 1].item->num_chars;
+					i ++;
+				} else
+					break;
+			}
+
+			pango_break (translated + tmp_item.offset, tmp_item.length, &tmp_item.analysis, text->pi->attrs + start_offset, tmp_item.num_chars + 1);
+		}
+
+		html_text_remove_link_line_breaks (text);
+
+		for (i = 0; i < text->pi->n; i ++) {
 			PangoGlyphString *glyphs;
 			PangoItem *item;
-			item = text->pi->entries [i].item = (PangoItem *) cur->data;
+
+			item = text->pi->entries [i].item;
 
 			/* printf ("item pos %d len %d\n", item->offset, item->length); */
 
-			text->pi->entries [i].attrs = g_new (PangoLogAttr, item->num_chars + 1);
-			pango_break (translated + item->offset, item->length, &item->analysis, text->pi->entries [i].attrs, item->num_chars + 1);
-
 			glyphs = pango_glyph_string_new ();
 			text->pi->entries [i].widths = g_new (PangoGlyphUnit, item->num_chars);
 			pango_shape (translated + item->offset, item->length, &item->analysis, glyphs);
@@ -1292,12 +1346,14 @@ gint
 html_text_tail_white_space (HTMLText *text, HTMLPainter *painter, gint offset, gint ii, gint io, gint *white_len, gint line_offset, gchar *s)
 {
 	HTMLTextPangoInfo *pi = html_text_get_pango_info (text, painter);
-	gint wl = 0;
-	gint ww = 0;
+	int wl = 0;
+	int ww = 0;
+	int current_offset = offset;
 
 	if (html_text_pi_backward (pi, &ii, &io)) {
 		s = g_utf8_prev_char (s);
-		if (pi->entries [ii].attrs [io].is_white) {
+		current_offset --;
+		if (pi->attrs [current_offset].is_white) {
 			if (HTML_IS_GDK_PAINTER (painter) || HTML_IS_PLAIN_PAINTER (painter)) {
 				if (*s == '\t' && offset > 1) {
 					gint skip = 8, co = offset - 1;
@@ -1318,10 +1374,11 @@ html_text_tail_white_space (HTMLText *text, HTMLPainter *painter, gint offset, g
 		}
 	}
 
-	if (!HTML_IS_GDK_PAINTER (painter) && !HTML_IS_PLAIN_PAINTER (painter) && wl)
-		html_text_calc_text_size (text, painter, html_text_get_text (text, offset - wl),
-					  wl, NULL, NULL, 0, &line_offset, html_text_get_font_style (text), text->face,
+	if (!HTML_IS_GDK_PAINTER (painter) && !HTML_IS_PLAIN_PAINTER (painter) && wl) {
+		html_text_calc_text_size (text, painter, html_text_get_text (text, offset - wl) - text->text,
+					  wl, NULL, NULL, &line_offset, html_text_get_font_style (text), text->face,
 					  &ww, NULL, NULL);
+	}
 
 	if (white_len)
 		*white_len = wl;
@@ -1333,8 +1390,8 @@ static void
 update_mw (HTMLText *text, HTMLPainter *painter, gint offset, gint *last_offset, gint *ww, gint *mw, gint ii, gint io, gchar *s, gint line_offset) {
 	if (!HTML_IS_GDK_PAINTER (painter) && !HTML_IS_PLAIN_PAINTER (painter)) {
 		gint w;
-		html_text_calc_text_size (text, painter, html_text_get_text (text, *last_offset),
-					  offset - *last_offset, NULL, NULL, 0, NULL, html_text_get_font_style (text), text->face,
+		html_text_calc_text_size (text, painter, html_text_get_text (text, *last_offset) - text->text,
+					  offset - *last_offset, NULL, NULL, NULL, html_text_get_font_style (text), text->face,
 					  &w, NULL, NULL);
 		*ww += w;
 	}
@@ -1347,10 +1404,9 @@ update_mw (HTMLText *text, HTMLPainter *painter, gint offset, gint *last_offset,
 }
 
 gboolean
-html_text_is_line_break (PangoItem *item, PangoLogAttr *attrs, int offset)
+html_text_is_line_break (PangoLogAttr attr)
 {
-	return (attrs [offset].is_line_break
-		&& (item->analysis.lang_engine != NULL || (offset > 0 && attrs [offset - 1].is_white)));
+	return attr.is_line_break;
 }
 
 static gint
@@ -1371,7 +1427,7 @@ calc_min_width (HTMLObject *self, HTMLPainter *painter)
 	while (offset < text->text_len) {
 		gint skip;
 
-		if (offset > 0 && html_text_is_line_break (pi->entries [ii].item, pi->entries [ii].attrs, io))
+		if (offset > 0 && html_text_is_line_break (pi->attrs [offset]))
 			update_mw (text, painter, offset, &last_offset, &ww, &mw, ii, io, s, line_offset);
 
 		if (*s == '\t') {
@@ -1384,8 +1440,6 @@ calc_min_width (HTMLObject *self, HTMLPainter *painter)
 				ww += PANGO_PIXELS (pi->entries [ii].widths [io]);
 			line_offset ++;
 		}
-		if (offset > 0 && ii != pi->n - 1 && io == pi->entries [ii].item->num_chars - 1)
-			update_mw (text, painter, offset, &last_offset, &ww, &mw, ii, io, s, line_offset);
 
 		s = g_utf8_next_char (s);
 		offset ++;
@@ -1395,8 +1449,8 @@ calc_min_width (HTMLObject *self, HTMLPainter *painter)
 
 	
 	if (!HTML_IS_GDK_PAINTER (painter) && !HTML_IS_PLAIN_PAINTER (painter))
-		html_text_calc_text_size (text, painter, html_text_get_text (text, last_offset),
-					  offset - last_offset, NULL, NULL, 0, NULL, html_text_get_font_style (text), text->face,
+		html_text_calc_text_size (text, painter, html_text_get_text (text, last_offset) - text->text,
+					  offset - last_offset, NULL, NULL, NULL, html_text_get_font_style (text), text->face,
 					  &ww, NULL, NULL);
 
 	if (ww > mw)
@@ -1631,8 +1685,18 @@ save_plain (HTMLObject *self,
 
 	text = HTML_TEXT (self);
 
+	if (text->links) {
+		GSList *cur;
+		int len = html_engine_save_buffer_peek_text_len (state);
+
+		for (cur = text->links; cur; cur = cur->next) {
+			Link *link = (Link *) cur->data;
+
+			html_engine_save_add_nb_interval (state, link->start_offset + len, link->end_offset + len);
+		}
+	}
 	rv  = html_engine_save_output_string (state, "%s", text->text);
-	
+
 	return rv;
 }
 
