@@ -35,6 +35,8 @@
 #include "htmlprinter.h"
 #include "htmlplainpainter.h"
 #include "htmlgdkpainter.h"
+#include "htmlsettings.h"
+#include "gtkhtml.h"
 
 
 /* #define HTML_TEXT_SLAVE_DEBUG */
@@ -138,8 +140,8 @@ get_offset_for_bounded_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *
 	char *buffer = html_text_slave_get_text (slave);
 
 	len = (lower + upper) / 2;
-	html_text_calc_text_size (slave->owner, painter, buffer, len, html_text_get_pango_info (text, painter),
-				  get_glyphs (slave, painter), html_text_slave_get_start_byte_offset (slave),
+	html_text_calc_text_size (slave->owner, painter, buffer - slave->owner->text, len, html_text_get_pango_info (text, painter),
+				  get_glyphs (slave, painter),
 				  &line_offset, html_text_get_font_style (text), text->face, &width, &asc, &dsc);
 	while (lower < upper) {
 		if (width > max_width)
@@ -148,8 +150,7 @@ get_offset_for_bounded_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *
 			lower = len + 1;
 		len = (lower + upper) / 2;
 		line_offset = -1;
-		html_text_calc_text_size (slave->owner, painter, buffer, len, html_text_get_pango_info (text, painter), get_glyphs (slave, painter),
-					  html_text_slave_get_start_byte_offset (slave),
+		html_text_calc_text_size (slave->owner, painter, buffer - slave->owner->text, len, html_text_get_pango_info (text, painter), get_glyphs (slave, painter),
 					  &line_offset, html_text_get_font_style (text), text->face, &width, &asc, &dsc);
 	}
 
@@ -322,15 +323,15 @@ update_lb (HTMLTextSlave *slave, HTMLPainter *painter, gint widthLeft, gint offs
 	if (HTML_IS_GDK_PAINTER (painter) || HTML_IS_PLAIN_PAINTER (painter)) {
 		aw = *w - new_ltw;
 	} else {
-		gint lo = html_text_get_line_offset (slave->owner, painter, *lbo - *lwl);
+		gint lo = html_text_get_line_offset (slave->owner, painter, *lbo);
+		gint width;
 				/* printf ("s: %s l: %d\n", html_text_get_text (slave->owner, lbo - lwl), offset - new_lwl - lbo + lwl); */
-		html_text_calc_text_size (slave->owner, painter, html_text_get_text (slave->owner, *lbo - *lwl),
-					  offset - new_lwl - *lbo + *lwl, NULL, NULL, 0, &lo,
+		html_text_calc_text_size (slave->owner, painter, html_text_get_text (slave->owner, *lbo) - slave->owner->text,
+					  offset - *lbo, NULL, NULL, &lo,
 					  html_text_get_font_style (slave->owner), slave->owner->face,
-					  &aw, NULL, NULL);
-		*w += aw;
-		aw = *w;
-		new_ltw = 0;
+					  &width, NULL, NULL);
+		*w += width;
+		aw = *w - new_ltw;
 	}
 	if (aw <= widthLeft || *force_fit) {
 		*ltw = new_ltw;
@@ -370,7 +371,7 @@ hts_fit_line (HTMLObject *o, HTMLPainter *painter,
 	lbsp = s = html_text_slave_get_text (slave);
 
 	while ((force_fit || widthLeft > lbw) && offset < slave->posStart + slave->posLen) {
-		if (offset > slave->posStart && offset > lbo && html_text_is_line_break (pi->entries [ii].item, pi->entries [ii].attrs, io))
+		if (offset > slave->posStart && offset > lbo && html_text_is_line_break (pi->attrs [offset]))
 			if (update_lb (slave, painter, widthLeft, offset, s, ii, io, line_offset, &w, &ltw, &lwl, &lbw, &lbo, &lbsp, &force_fit))
 				break;
 
@@ -388,20 +389,16 @@ hts_fit_line (HTMLObject *o, HTMLPainter *painter,
 		s = g_utf8_next_char (s);
 		offset ++;
 
-		if (offset > slave->posStart && ii != pi->n - 1 && io == pi->entries [ii].item->num_chars - 1)
-			if (update_lb (slave, painter, widthLeft, offset, s, ii, io, line_offset, &w, &ltw, &lwl, &lbw, &lbo, &lbsp, &force_fit))
-				break;
-
 		html_text_pi_forward (pi, &ii, &io);
 	}
 
 	if (!HTML_IS_GDK_PAINTER (painter) && !HTML_IS_PLAIN_PAINTER (painter)) {
 		gint aw;
-		gint lo = html_text_get_line_offset (slave->owner, painter, lbo - lwl);
+		gint lo = html_text_get_line_offset (slave->owner, painter, lbo);
 
 		/* printf ("s: %s l: %d\n", html_text_get_text (slave->owner, lbo - lwl), offset - lbo + lwl); */
-		html_text_calc_text_size (slave->owner, painter, html_text_get_text (slave->owner, lbo - lwl),
-					  offset - lbo + lwl, NULL, NULL, 0, &lo,
+		html_text_calc_text_size (slave->owner, painter, html_text_get_text (slave->owner, lbo) - slave->owner->text,
+					  offset - lbo, NULL, NULL, &lo,
 					  html_text_get_font_style (slave->owner), slave->owner->face,
 					  &aw, NULL, NULL);
 		w += aw;
@@ -465,6 +462,12 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 	gint last_off = 0;
 	gint line_offset = html_text_slave_get_line_offset (slave, 0, p);
 	gchar *text = html_text_slave_get_text (slave);
+	HTMLEngine *e;
+
+	if (p->widget && GTK_IS_HTML (p->widget))
+		e = GTK_HTML (p->widget)->engine;
+	else
+		return;
 
 	while (cur) {
 
@@ -477,13 +480,14 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 			guint len = mi - ma;
 			gint lo, width, asc, dsc;
 
-			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLSpellErrorColor)->color);
+			html_painter_set_pen (p, &html_colorset_get_color_allocated (e->settings->color_set,
+										     p, HTMLSpellErrorColor)->color);
 			/* printf ("spell error: %s\n", html_text_get_text (slave->owner, off)); */
 			lo = line_offset;
 			
 			glyphs = get_glyphs_part (slave, p, last_off, off - last_off);
-			html_text_calc_text_size (slave->owner, p, text,
-						  off - last_off, html_text_get_pango_info (slave->owner, p), glyphs, text - slave->owner->text,
+			html_text_calc_text_size (slave->owner, p, text - slave->owner->text,
+						  off - last_off, html_text_get_pango_info (slave->owner, p), glyphs,
 						  &line_offset,
 						  p->font_style,
 						  p->font_face, &width, &asc, &dsc);
@@ -637,8 +641,8 @@ draw_normal (HTMLTextSlave *self,
 			gchar *text = html_text_slave_get_text (self);
 			gint start_index, end_index;
 
-			start_index = html_text_slave_get_text (self) - self->owner->text;
-			end_index = g_utf8_offset_to_pointer (text, self->posLen) - text;
+			start_index = text - self->owner->text;
+			end_index = g_utf8_offset_to_pointer (text, self->posLen) - self->owner->text;
 			attrs = html_text_get_attr_list (self->owner, start_index, end_index);
 		}
 
@@ -659,6 +663,12 @@ draw_focus  (HTMLPainter *painter, GdkRectangle *box)
 	HTMLGdkPainter *p;
 	GdkGCValues values;
 	gchar dash [2];
+	HTMLEngine *e;
+
+	if (painter->widget && GTK_IS_HTML (painter->widget))
+		e = GTK_HTML (painter->widget)->engine;
+	else
+		return;
 
 	if (HTML_IS_PRINTER (painter))
 		return;
@@ -666,7 +676,8 @@ draw_focus  (HTMLPainter *painter, GdkRectangle *box)
 	p = HTML_GDK_PAINTER (painter);
 	/* printf ("draw_text_focus\n"); */
 
-	gdk_gc_set_foreground (p->gc, &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
+	gdk_gc_set_foreground (p->gc, &html_colorset_get_color_allocated (e->settings->color_set,
+									  painter, HTMLTextColor)->color);
 	gdk_gc_get_values (p->gc, &values);
 
 	dash [0] = 1;
@@ -791,8 +802,7 @@ calc_offset (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint y)
 
 		if (len) {
 			glyphs = get_glyphs_part (slave, painter, 0, len);
-			html_text_calc_text_size (slave->owner, painter, text, len, html_text_get_pango_info (owner, painter), glyphs,
-						  html_text_slave_get_start_byte_offset (slave), 
+			html_text_calc_text_size (slave->owner, painter, text - slave->owner->text, len, html_text_get_pango_info (owner, painter), glyphs,
 						  &lo, font_style, owner->face, &width, &asc, &dsc);
 			glyphs_destroy (glyphs);
 		} else {
